# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-09 20:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0002_auto_20160907_1914'),
    ]

    operations = [
        migrations.AddField(
            model_name='authservicesinfo',
            name='state',
            field=models.CharField(blank=True, choices=[(None, b'None'), (b'Blue', b'Blue'), (b'Member', b'Member')], default=None, max_length=10, null=True),
        ),
    ]



from django.contrib.auth.models import User
from django.db import models
from sortedm2m.fields import SortedManyToManyField

from allianceauth.eveonline.models import EveCharacter
from allianceauth.eveonline.models import EveCorporationInfo


class ApplicationQuestion(models.Model):
    title = models.CharField(max_length=254, verbose_name='Question')
    help_text = models.CharField(max_length=254, blank=True, null=True)
    multi_select = models.BooleanField(default=False)

    def __str__(self):
        return "Question: " + self.title


class ApplicationChoice(models.Model):
    question = models.ForeignKey(ApplicationQuestion,on_delete=models.CASCADE,related_name="choices")
    choice_text = models.CharField(max_length=200, verbose_name='Choice')

    def __str__(self):
        return self.choice_text

class ApplicationForm(models.Model):
    questions = SortedManyToManyField(ApplicationQuestion)
    corp = models.OneToOneField(EveCorporationInfo, on_delete=models.CASCADE)

    def __str__(self):
        return str(self.corp)


class Application(models.Model):
    form = models.ForeignKey(ApplicationForm, on_delete=models.CASCADE, related_name='applications')
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='applications')
    approved = models.NullBooleanField(blank=True, null=True, default=None)
    reviewer = models.ForeignKey(User, on_delete=models.SET_NULL, blank=True, null=True)
    reviewer_character = models.ForeignKey(EveCharacter, on_delete=models.SET_NULL, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return str(self.user) + " Application To " + str(self.form)

    class Meta:
        permissions = (
            ('approve_application', 'Can approve applications'), ('reject_application', 'Can reject applications'),
            ('view_apis', 'Can view applicant APIs'),)
        unique_together = ('form', 'user')

    @property
    def main_character(self):
        return self.user.profile.main_character

    @property
    def characters(self):
        return [o.character for o in self.user.character_ownerships.all()]

    @property
    def reviewer_str(self):
        if self.reviewer_character:
            return str(self.reviewer_character)
        elif self.reviewer:
            return "User " + str(self.reviewer)
        else:
            return None


class ApplicationResponse(models.Model):
    question = models.ForeignKey(ApplicationQuestion, on_delete=models.CASCADE)
    application = models.ForeignKey(Application, on_delete=models.CASCADE, related_name='responses')
    answer = models.TextField()

    def __str__(self):
        return str(self.application) + " Answer To " + str(self.question)

    class Meta:
        unique_together = ('question', 'application')


class ApplicationComment(models.Model):
    application = models.ForeignKey(Application, on_delete=models.CASCADE, related_name='comments')
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    text = models.TextField()
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return str(self.user) + " comment on " + str(self.application)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-09 23:11
from __future__ import unicode_literals

from django.db import migrations

def delete_permissions(apps, schema_editor):
    User = apps.get_model('auth', 'User')
    ContentType = apps.get_model('contenttypes', 'ContentType')
    Permission = apps.get_model('auth', 'Permission')
    ct = ContentType.objects.get_for_model(User)
    Permission.objects.filter(content_type=ct).filter(codename__in=['member', 'blue_member', 'signature_management', 'signature_view']).delete()

def create_permissions(apps, schema_editor):
    User = apps.get_model('auth', 'User')
    ContentType = apps.get_model('contenttypes', 'ContentType')
    Permission = apps.get_model('auth', 'Permission')
    ct = ContentType.objects.get_for_model(User)
    Permission.objects.get_or_create(codename="member", content_type=ct, name="member")
    Permission.objects.get_or_create(codename="blue_member", content_type=ct, name="blue_member") 


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0004_create_permissions'),
    ]

    operations = [
        migrations.RunPython(delete_permissions, create_permissions),
    ]



import logging

from django.core.exceptions import ObjectDoesNotExist

from allianceauth.notifications import notify
from allianceauth.services.hooks import NameFormatter
from .manager import XenForoManager
from .models import XenforoUser

logger = logging.getLogger(__name__)


class XenforoTasks:
    def __init__(self):
        pass

    @classmethod
    def delete_user(cls, user, notify_user=False):
        if cls.has_account(user):
            logger.debug("User %s has a XenForo account %s. Deleting." % (user, user.xenforo.username))
            if XenForoManager.disable_user(user.xenforo.username) == 200:
                user.xenforo.delete()
                if notify_user:
                    notify(user, 'XenForo Account Disabled', level='danger')
                return True
        return False

    @staticmethod
    def has_account(user):
        try:
            return user.xenforo.username != ''
        except ObjectDoesNotExist:
            return False

    @classmethod
    def disable(cls):
        logger.debug("Deleting ALL XenForo users")
        XenforoUser.objects.all().delete()

    @staticmethod
    def get_username(user):
        from .auth_hooks import XenforoService
        return NameFormatter(XenforoService(), user).format_name()



from django.conf.urls import include
from functools import wraps
from django.shortcuts import redirect
from django.contrib import messages
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.decorators import login_required


def user_has_main_character(user):
    return bool(user.profile.main_character)


def decorate_url_patterns(urls, decorator):
    url_list, app_name, namespace = include(urls)

    def process_patterns(url_patterns):
        for pattern in url_patterns:
            if hasattr(pattern, 'url_patterns'):
                # this is an include - apply to all nested patterns
                process_patterns(pattern.url_patterns)
            else:
                # this is a pattern
                pattern.callback = decorator(pattern.callback)

    process_patterns(url_list)
    return url_list, app_name, namespace


def main_character_required(view_func):
    @wraps(view_func)
    def _wrapped_view(request, *args, **kwargs):
        if user_has_main_character(request.user):
            return view_func(request, *args, **kwargs)

        messages.error(request, _('A main character is required to perform that action. Add one below.'))
        return redirect('authentication:dashboard')
    return login_required(_wrapped_view)



from django.conf import settings
from django.core.cache import cache
from django.utils import timezone
from datetime import datetime

import logging
import requests
import hashlib

logger = logging.getLogger(__name__)


class FleetUpManager:
    APP_KEY = settings.FLEETUP_APP_KEY
    USER_ID = settings.FLEETUP_USER_ID
    API_ID = settings.FLEETUP_API_ID
    GROUP_ID = settings.FLEETUP_GROUP_ID
    BASE_URL = "http://api.fleet-up.com/Api.svc/{}/{}/{}".format(APP_KEY, USER_ID, API_ID)

    TZ = timezone.utc

    def __init__(self):
        pass

    @classmethod
    def _request_cache_key(cls, url):
        h = hashlib.sha1()
        h.update(url.encode('utf-8'))
        return 'FLEETUP_ENDPOINT_' + h.hexdigest()

    @classmethod
    def _cache_until_seconds(cls, cache_until_json):
        # Format comes in like "/Date(1493896236163)/"
        try:
            epoch_ms = int(cache_until_json[6:-2])
            cache_delta = datetime.fromtimestamp(epoch_ms/1000) - datetime.now()
            cache_delta_seconds = cache_delta.total_seconds()
            if cache_delta_seconds < 0:
                return 0
            elif cache_delta_seconds > 3600:
                return 3600
            else:
                return cache_delta_seconds
        except TypeError:
            logger.debug("Couldn't convert CachedUntil time, defaulting to 600 seconds")
        return 600

    @classmethod
    def get_endpoint(cls, url):
        try:
            cache_key = cls._request_cache_key(url)
            cached = cache.get(cache_key)
            if cached:
                return cached

            r = requests.get(url)
            r.raise_for_status()

            json = r.json()

            if json['Success']:
                cache.set(cache_key, json, cls._cache_until_seconds(json['CachedUntilUTC']))
            return json
        except requests.exceptions.ConnectionError:
            logger.warning("Can't connect to Fleet-Up API, is it offline?!")
        except requests.HTTPError:
            logger.exception("Error accessing Fleetup API")
        return None

    @classmethod
    def get_fleetup_members(cls):
        url = "{}/GroupCharacters/{}".format(cls.BASE_URL, cls.GROUP_ID)
        try:
            fmembers = cls.get_endpoint(url)
            if not fmembers:
                return None
            return {row["UserId"]: {"user_id": row["UserId"],
                                    "char_name": row["EveCharName"],
                                    "char_id": row["EveCharId"],
                                    "corporation": row["Corporation"]} for row in fmembers["Data"]}
        except (ValueError, UnicodeDecodeError, TypeError):
            logger.debug("No fleetup members retrieved.")
        return {}

    @classmethod
    def get_fleetup_operations(cls):
        url = "{}/Operations/{}".format(cls.BASE_URL, cls.GROUP_ID)
        foperations = cls.get_endpoint(url)
        if foperations is None:
            return None
        return {row["StartString"]: {"subject": row["Subject"],
                                     "start": timezone.make_aware(
                                         datetime.strptime(row["StartString"], "%Y-%m-%d %H:%M:%S"), cls.TZ),
                                     "end": timezone.make_aware(
                                         datetime.strptime(row["EndString"], "%Y-%m-%d %H:%M:%S"), cls.TZ),
                                     "operation_id": row["OperationId"],
                                     "location": row["Location"],
                                     "location_info": row["LocationInfo"],
                                     "details": row["Details"],
                                     "url": row["Url"],
                                     "doctrine": row["Doctrines"],
                                     "organizer": row["Organizer"]} for row in foperations["Data"]}

    @classmethod
    def get_fleetup_timers(cls):
        url = "{}/Timers/{}".format(cls.BASE_URL, cls.GROUP_ID)
        ftimers = cls.get_endpoint(url)
        if not ftimers:
            return None
        return {row["ExpiresString"]: {"solarsystem": row["SolarSystem"],
                                       "planet": row["Planet"],
                                       "moon": row["Moon"],
                                       "owner": row["Owner"],
                                       "type": row["Type"],
                                       "timer_type": row["TimerType"],
                                       "expires": timezone.make_aware(
                                           datetime.strptime(row["ExpiresString"], "%Y-%m-%d %H:%M:%S"), cls.TZ),
                                       "notes": row["Notes"]} for row in ftimers["Data"]}

    @classmethod
    def get_fleetup_doctrines(cls):
        url = "{}/Doctrines/{}".format(cls.BASE_URL, cls.GROUP_ID)
        fdoctrines = cls.get_endpoint(url)
        if not fdoctrines:
            return None
        return {"fleetup_doctrines": fdoctrines["Data"]}

    @classmethod
    def get_fleetup_doctrine(cls, doctrinenumber):
        url = "{}/DoctrineFittings/{}".format(cls.BASE_URL, doctrinenumber)
        fdoctrine = cls.get_endpoint(url)
        if not fdoctrine:
            return None
        return {"fitting_doctrine": fdoctrine}

    @classmethod
    def get_fleetup_fittings(cls):
        url = "{}/Fittings/{}".format(cls.BASE_URL, cls.GROUP_ID)
        ffittings = cls.get_endpoint(url)
        if not ffittings:
            return None
        return {row["FittingId"]: {"fitting_id": row["FittingId"],
                                   "name": row["Name"],
                                   "icon_id": row["EveTypeId"],
                                   "hull": row["HullType"],
                                   "shiptype": row["ShipType"],
                                   "estimated": row["EstPrice"],
                                   "faction": row["Faction"],
                                   "categories": row["Categories"],
                                   "last_update":
                                       timezone.make_aware(
                                           datetime.strptime(row["LastUpdatedString"], "%Y-%m-%d %H:%M:%S"), cls.TZ)}
                for row in ffittings["Data"]}

    @classmethod
    def get_fleetup_fitting(cls, fittingnumber):
        url = "{}/Fitting/{}".format(cls.BASE_URL, fittingnumber)
        try:
            ffitting = cls.get_endpoint(url)
            if not ffitting:
                return None
            return {"fitting_data": ffitting["Data"]}
        except KeyError:
            logger.warning("Failed to retrieve fleetup fitting number %s" % fittingnumber)
        return {"fitting_data": {}}

    @classmethod
    def get_fleetup_doctrineid(cls, fittingnumber):
        url = "{}/Fitting/{}".format(cls.BASE_URL, fittingnumber)
        try:
            fdoctrineid = cls.get_endpoint(url)
            if not fdoctrineid:
                return None
            return fdoctrineid['Data']['Doctrines'][0]['DoctrineId']
        except (KeyError, IndexError):
            logger.debug("Fleetup fitting number %s not in a doctrine." % fittingnumber)
        return {}

    @classmethod
    def get_fleetup_fitting_eft(cls, fittingnumber):
        url = "{}/Fitting/{}/eft".format(cls.BASE_URL, fittingnumber)
        try:
            ffittingeft = cls.get_endpoint(url)
            if not ffittingeft:
                return None
            return {"fitting_eft": ffittingeft["Data"]["FittingData"]}
        except KeyError:
            logger.warning("Fleetup fitting eft not found for fitting number %s" % fittingnumber)
        return {"fitting_eft": {}}



from django.contrib import admin
from django import forms
from allianceauth import hooks
from .models import NameFormatConfig


class NameFormatConfigForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(NameFormatConfigForm, self).__init__(*args, **kwargs)
        SERVICE_CHOICES = [(s.name, s.name) for h in hooks.get_hooks('services_hook') for s in [h()]]
        if self.instance.id:
            current_choice = (self.instance.service_name, self.instance.service_name)
            if current_choice not in SERVICE_CHOICES:
                SERVICE_CHOICES.append(current_choice)
        self.fields['service_name'] = forms.ChoiceField(choices=SERVICE_CHOICES)


class NameFormatConfigAdmin(admin.ModelAdmin):
    form = NameFormatConfigForm
    list_display = ('service_name', 'get_state_display_string')

    def get_state_display_string(self, obj):
        return ', '.join([state.name for state in obj.states.all()])
    get_state_display_string.short_description = 'States'


admin.site.register(NameFormatConfig, NameFormatConfigAdmin)



import random
import string
import calendar
from datetime import datetime
import hashlib
import logging
import re

from django.db import connections
from django.conf import settings

logger = logging.getLogger(__name__)


TABLE_PREFIX = getattr(settings, 'SMF_TABLE_PREFIX', 'smf_')


class SmfManager:
    def __init__(self):
        pass

    SQL_ADD_USER = r"INSERT INTO %smembers (member_name, passwd, email_address, date_registered, real_name," \
                   r" buddy_list, message_labels, openid_uri, signature, ignore_boards) " \
                   r"VALUES (%%s, %%s, %%s, %%s, %%s, 0, 0, 0, 0, 0)" % TABLE_PREFIX

    SQL_DEL_USER = r"DELETE FROM %smembers where member_name = %%s" % TABLE_PREFIX

    SQL_DIS_USER = r"UPDATE %smembers SET email_address = %%s, passwd = %%s WHERE member_name = %%s" % TABLE_PREFIX

    SQL_USER_ID_FROM_USERNAME = r"SELECT id_member from %smembers WHERE member_name = %%s" % TABLE_PREFIX

    SQL_ADD_USER_GROUP = r"UPDATE %smembers SET additional_groups = %%s WHERE id_member = %%s" % TABLE_PREFIX

    SQL_GET_GROUP_ID = r"SELECT id_group from %smembergroups WHERE group_name = %%s" % TABLE_PREFIX

    SQL_ADD_GROUP = r"INSERT INTO %smembergroups (group_name,description) VALUES (%%s,%%s)" % TABLE_PREFIX

    SQL_UPDATE_USER_PASSWORD = r"UPDATE %smembers SET passwd = %%s WHERE member_name = %%s" % TABLE_PREFIX

    SQL_REMOVE_USER_GROUP = r"UPDATE %smembers SET additional_groups = %%s WHERE id_member = %%s" % TABLE_PREFIX

    SQL_GET_ALL_GROUPS = r"SELECT id_group, group_name FROM %smembergroups" % TABLE_PREFIX

    SQL_GET_USER_GROUPS = r"SELECT additional_groups FROM %smembers WHERE id_member = %%s" % TABLE_PREFIX

    SQL_ADD_USER_AVATAR = r"UPDATE %smembers SET avatar = %%s WHERE id_member = %%s" % TABLE_PREFIX

    @staticmethod
    def _sanitize_groupname(name):
        name = name.strip(' _')
        return re.sub('[^\w.-]', '', name)

    @staticmethod
    def generate_random_pass():
        return ''.join([random.choice(string.ascii_letters + string.digits) for n in range(16)])

    @staticmethod
    def gen_hash(username_clean, passwd):
        return hashlib.sha1((username_clean + passwd).encode('utf-8')).hexdigest()

    @staticmethod
    def santatize_username(username):
        sanatized = username.replace(" ", "_")
        sanatized = sanatized.replace("'", "_")
        return sanatized.lower()

    @staticmethod
    def get_current_utc_date():
        d = datetime.utcnow()
        unixtime = calendar.timegm(d.utctimetuple())
        return unixtime

    @classmethod
    def create_group(cls, groupname):
        logger.debug("Creating smf group %s" % groupname)
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_ADD_GROUP, [groupname, groupname])
        logger.info("Created smf group %s" % groupname)
        return cls.get_group_id(groupname)

    @classmethod
    def get_group_id(cls, groupname):
        logger.debug("Getting smf group id for groupname %s" % groupname)
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_GET_GROUP_ID, [groupname])
        row = cursor.fetchone()
        logger.debug("Got smf group id %s for groupname %s" % (row[0], groupname))
        return row[0]

    @classmethod
    def check_user(cls, username):
        logger.debug("Checking smf username %s" % username)
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_USER_ID_FROM_USERNAME, [cls.santatize_username(username)])
        row = cursor.fetchone()
        if row:
            logger.debug("Found user %s on smf" % username)
            return True
        logger.debug("User %s not found on smf" % username)
        return False

    @classmethod
    def add_avatar(cls, member_name, characterid):
        logger.debug("Adding EVE character id %s portrait as smf avatar for user %s" % (characterid, member_name))
        avatar_url = "https://image.eveonline.com/Character/" + characterid + "_64.jpg"
        cursor = connections['smf'].cursor()
        id_member = cls.get_user_id(member_name)
        cursor.execute(cls.SQL_ADD_USER_AVATAR, [avatar_url, id_member])

    @classmethod
    def get_user_id(cls, username):
        logger.debug("Getting smf user id for username %s" % username)
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_USER_ID_FROM_USERNAME, [username])
        row = cursor.fetchone()
        if row is not None:
            logger.debug("Got smf user id %s for username %s" % (row[0], username))
            return row[0]
        else:
            logger.error("username %s not found on smf. Unable to determine user id ." % username)
            return None

    @classmethod
    def get_all_groups(cls):
        logger.debug("Getting all smf groups.")
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_GET_ALL_GROUPS)
        rows = cursor.fetchall()
        out = {}
        for row in rows:
            out[row[1]] = row[0]
        logger.debug("Got smf groups %s" % out)
        return out

    @classmethod
    def get_user_groups(cls, userid):
        logger.debug("Getting smf user id %s groups" % userid)
        cursor = connections['smf'].cursor()
        cursor.execute(cls.SQL_GET_USER_GROUPS, [userid])
        out = [row[0] for row in cursor.fetchall()]
        logger.debug("Got user %s smf groups %s" % (userid, out))
        return out

    @classmethod
    def add_user(cls, username, email_address, groups, characterid):
        logger.debug("Adding smf user with member_name %s, email_address %s, characterid %s" % (
            username, email_address, characterid))
        cursor = connections['smf'].cursor()
        username_clean = cls.santatize_username(username)
        passwd = cls.generate_random_pass()
        pwhash = cls.gen_hash(username_clean, passwd)
        logger.debug("Proceeding to add smf user %s and pwhash starting with %s" % (username, pwhash[0:5]))
        register_date = cls.get_current_utc_date()
        # check if the username was simply revoked
        if cls.check_user(username) is True:
            logger.warn("Unable to add smf user with username %s - already exists. Updating user instead." % username)
            cls.__update_user_info(username_clean, email_address, pwhash)
        else:
            try:
                cursor.execute(cls.SQL_ADD_USER,
                               [username_clean, passwd, email_address, register_date, username_clean])
                cls.add_avatar(username_clean, characterid)
                logger.info("Added smf member_name %s" % username_clean)
                cls.update_groups(username_clean, groups)
            except:
                logger.warn("Unable to add smf user %s" % username_clean)
                pass
        return username_clean, passwd

    @classmethod
    def __update_user_info(cls, username, email_address, passwd):
        logger.debug(
            "Updating smf user %s info: username %s password of length %s" % (username, email_address, len(passwd)))
        cursor = connections['smf'].cursor()
        try:
            cursor.execute(cls.SQL_DIS_USER, [email_address, passwd, username])
            logger.info("Updated smf user %s info" % username)
        except:
            logger.exception("Unable to update smf user %s info." % username)
            pass

    @classmethod
    def delete_user(cls, username):
        logger.debug("Deleting smf user %s" % username)
        cursor = connections['smf'].cursor()

        if cls.check_user(username):
            cursor.execute(cls.SQL_DEL_USER, [username])
            logger.info("Deleted smf user %s" % username)
            return True
        logger.error("Unable to delete smf user %s - user not found on smf." % username)
        return False

    @classmethod
    def update_groups(cls, username, groups):
        userid = cls.get_user_id(username)
        logger.debug("Updating smf user %s with id %s groups %s" % (username, userid, groups))
        if userid is not None:
            forum_groups = cls.get_all_groups()
            user_groups = set(cls.get_user_groups(userid))
            act_groups = set([cls._sanitize_groupname(g) for g in groups])
            addgroups = act_groups - user_groups
            remgroups = user_groups - act_groups
            logger.info("Updating smf user %s groups - adding %s, removing %s" % (username, addgroups, remgroups))
            act_group_id = set()
            for g in addgroups:
                if g not in forum_groups:
                    forum_groups[g] = cls.create_group(g)
                act_group_id.add(str(cls.get_group_id(g)))
            string_groups = ','.join(act_group_id)
            cls.add_user_to_group(userid, string_groups)

    @classmethod
    def add_user_to_group(cls, userid, groupid):
        logger.debug("Adding smf user id %s to group id %s" % (userid, groupid))
        try:
            cursor = connections['smf'].cursor()
            cursor.execute(cls.SQL_ADD_USER_GROUP, [groupid, userid])
            logger.info("Added smf user id %s to group id %s" % (userid, groupid))
        except:
            logger.exception("Unable to add smf user id %s to group id %s" % (userid, groupid))
            pass

    @classmethod
    def remove_user_from_group(cls, userid, groupid):
        logger.debug("Removing smf user id %s from group id %s" % (userid, groupid))
        try:
            cursor = connections['smf'].cursor()
            cursor.execute(cls.SQL_REMOVE_USER_GROUP, [groupid, userid])
            logger.info("Removed smf user id %s from group id %s" % (userid, groupid))
        except:
            logger.exception("Unable to remove smf user id %s from group id %s" % (userid, groupid))
            pass

    @classmethod
    def disable_user(cls, username):
        logger.debug("Disabling smf user %s" % username)
        cursor = connections['smf'].cursor()

        password = cls.generate_random_pass()
        revoke_email = "revoked@localhost"
        try:
            pwhash = cls.gen_hash(username, password)
            cursor.execute(cls.SQL_DIS_USER, [revoke_email, pwhash, username])
            cls.get_user_id(username)
            cls.update_groups(username, [])
            logger.info("Disabled smf user %s" % username)
            return True
        except TypeError:
            logger.exception("TypeError occured while disabling user %s - failed to disable." % username)
            return False

    @classmethod
    def update_user_password(cls, username, characterid, password=None):
        logger.debug("Updating smf user %s password" % username)
        cursor = connections['smf'].cursor()
        if not password:
            password = cls.generate_random_pass()
        if cls.check_user(username):
            username_clean = cls.santatize_username(username)
            pwhash = cls.gen_hash(username_clean, password)
            logger.debug(
                "Proceeding to update smf user %s password with pwhash starting with %s" % (username, pwhash[0:5]))
            cursor.execute(cls.SQL_UPDATE_USER_PASSWORD, [pwhash, username])
            cls.add_avatar(username, characterid)
            logger.info("Updated smf user %s password." % username)
            return password
        logger.error("Unable to update smf user %s password - user not found on smf." % username)
        return ""



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, redirect

from allianceauth.services.forms import ServicePasswordForm
from .manager import MarketManager
from .models import MarketUser
from .tasks import MarketTasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'market.access_market'


@login_required
@permission_required(ACCESS_PERM)
def activate_market(request):
    logger.debug("activate_market called by user %s" % request.user)
    character = request.user.profile.main_character
    if character is not None:
        logger.debug("Adding market user for user %s with main character %s" % (request.user, character))
        result = MarketManager.add_user(character.character_name, request.user.email, character.character_id,
                                        character.character_name)
        # if empty we failed
        if result[0] != "":
            MarketUser.objects.create(user=request.user, username=result[0])
            logger.debug("Updated authserviceinfo for user %s with market credentials." % request.user)
            logger.info("Successfully activated market for user %s" % request.user)
            messages.success(request, 'Activated Alliance Market account.')
            credentials = {
                'username': result[0],
                'password': result[1],
            }
            return render(request, 'services/service_credentials.html',
                          context={'credentials': credentials, 'service': 'Alliance Market'})
    logger.error("Unsuccessful attempt to activate market for user %s" % request.user)
    messages.error(request, 'An error occurred while processing your Alliance Market account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def deactivate_market(request):
    logger.debug("deactivate_market called by user %s" % request.user)
    # false we failed
    if MarketTasks.delete_user(request.user):
        logger.info("Successfully deactivated market for user %s" % request.user)
        messages.success(request, 'Deactivated Alliance Market account.')
    else:
        logger.error("Unsuccessful attempt to activate market for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your Alliance Market account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_market_password(request):
    logger.debug("reset_market_password called by user %s" % request.user)
    if MarketTasks.has_account(request.user):
        result = MarketManager.update_user_password(request.user.market.username)
        # false we failed
        if result != "":
            logger.info("Successfully reset market password for user %s" % request.user)
            messages.success(request, 'Reset Alliance Market password.')
            credentials = {
                'username': request.user.market.username,
                'password': result,
            }
            return render(request, 'services/service_credentials.html',
                          context={'credentials': credentials, 'service': 'Alliance Market'})

    logger.error("Unsuccessful attempt to reset market password for user %s" % request.user)
    messages.error(request, 'An error occurred while processing your Alliance Market account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def set_market_password(request):
    logger.debug("set_market_password called by user %s" % request.user)
    if request.method == 'POST':
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug("Form is valid: %s" % form.is_valid())
        if form.is_valid() and MarketTasks.has_account(request.user):
            password = form.cleaned_data['password']
            logger.debug("Form contains password of length %s" % len(password))
            result = MarketManager.update_custom_password(request.user.market.username, password)
            if result != "":
                logger.info("Successfully reset market password for user %s" % request.user)
                messages.success(request, 'Set Alliance Market password.')
            else:
                logger.error("Failed to install custom market password for user %s" % request.user)
                messages.error(request, 'An error occurred while processing your Alliance Market account.')
            return redirect("services:services")
    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()

    logger.debug("Rendering form for user %s" % request.user)
    context = {'form': form, 'service': 'Market'}
    return render(request, 'services/service_password.html', context=context)



import requests
import logging
import amqp.exceptions
import semantic_version as semver
from django import template
from django.conf import settings
from django.core.cache import cache
from celery.app import app_or_default
from allianceauth import __version__

register = template.Library()

TAG_CACHE_TIME = 10800  # 3 hours
NOTIFICATION_CACHE_TIME = 300  # 5 minutes


logger = logging.getLogger(__name__)


def get_github_tags():
    request = requests.get('https://api.github.com/repos/allianceauth/allianceauth/releases')
    request.raise_for_status()
    return request.json()


def get_github_notification_issues():
    # notification
    request = requests.get(
        'https://api.github.com/repos/allianceauth/allianceauth/issues?labels=announcement&state=all')
    request.raise_for_status()
    return request.json()


@register.inclusion_tag('allianceauth/admin-status/overview.html', takes_context=True)
def status_overview(context):
    response = {
        'notifications': list(),
        'latest_major': True,
        'latest_minor': True,
        'latest_patch': True,
        'current_version': __version__,
        'task_queue_length': -1,
    }

    response.update(get_notifications())
    response.update(get_version_info())
    response.update({'task_queue_length': get_celery_queue_length()})

    return response


def get_celery_queue_length():
    try:
        app = app_or_default(None)
        with app.connection_or_acquire() as conn:
            return conn.default_channel.queue_declare(
                queue=getattr(settings, 'CELERY_DEFAULT_QUEUE', 'celery'), passive=True).message_count
    except amqp.exceptions.ChannelError:
        # Queue doesn't exist, probably empty
        return 0
    except Exception:
        logger.exception("Failed to get celery queue length")
    return -1


def get_notifications():
    response = {
        'notifications': list(),
    }
    try:
        notifications = cache.get_or_set('github_notification_issues', get_github_notification_issues,
                                         NOTIFICATION_CACHE_TIME)
        # Limit notifications to those posted by repo owners and members
        response['notifications'] += [n for n in notifications if n['author_association'] in ['OWNER', 'MEMBER']][:5]
    except requests.RequestException:
        logger.exception('Error while getting github notifications')
    return response


def get_version_info():
    response = {
        'latest_major': True,
        'latest_minor': True,
        'latest_patch': True,
        'current_version': __version__,
    }
    try:
        tags = cache.get_or_set('github_release_tags', get_github_tags, TAG_CACHE_TIME)
        current_ver = semver.Version.coerce(__version__)

        # Set them all to the current version to start
        # If the server has only earlier or the same version
        # then this will become the major/minor/patch versions
        latest_major = current_ver
        latest_minor = current_ver
        latest_patch = current_ver

        response.update({
            'latest_major_version': str(latest_major),
            'latest_minor_version': str(latest_minor),
            'latest_patch_version': str(latest_patch),
        })

        for tag in tags:
            tag_name = tag.get('tag_name')
            if tag_name[0] == 'v':
                # Strip 'v' off front of verison if it exists
                tag_name = tag_name[1:]
            try:
                tag_ver = semver.Version.coerce(tag_name)
            except ValueError:
                tag_ver = semver.Version('0.0.0', partial=True)
            if tag_ver > current_ver:
                if latest_major is None or tag_ver > latest_major:
                    latest_major = tag_ver
                    response['latest_major_version'] = tag_name
                    response['latest_major_url'] = tag['html_url']
                if tag_ver.major > current_ver.major:
                    response['latest_major'] = False
                elif tag_ver.major == current_ver.major:
                    if latest_minor is None or tag_ver > latest_minor:
                        latest_minor = tag_ver
                        response['latest_minor_version'] = tag_name
                        response['latest_minor_url'] = tag['html_url']
                    if tag_ver.minor > current_ver.minor:
                        response['latest_minor'] = False
                    elif tag_ver.minor == current_ver.minor:
                        if latest_patch is None or tag_ver > latest_patch:
                            latest_patch = tag_ver
                            response['latest_patch_version'] = tag_name
                            response['latest_patch_url'] = tag['html_url']
                        if tag_ver.patch > current_ver.patch:
                            response['latest_patch'] = False

    except requests.RequestException:
        logger.exception('Error while getting github release tags')
    return response



import logging

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from celery import shared_task
from allianceauth.services.tasks import QueueOnce
from allianceauth.notifications import notify
from allianceauth.services.hooks import NameFormatter
from .manager import Teamspeak3Manager
from .models import AuthTS, TSgroup, UserTSgroup, Teamspeak3User
from .util.ts3 import TeamspeakError

logger = logging.getLogger(__name__)


class Teamspeak3Tasks:
    def __init__(self):
        pass

    @classmethod
    def delete_user(cls, user, notify_user=False):
        if cls.has_account(user):
            logger.debug("User %s has TS3 account %s. Deleting." % (user, user.teamspeak3.uid))
            with Teamspeak3Manager() as ts3man:
                if ts3man.delete_user(user.teamspeak3.uid):
                    user.teamspeak3.delete()
                    if notify_user:
                        notify(user, 'TeamSpeak3 Account Disabled', level='danger')
                    return True
        return False

    @staticmethod
    def has_account(user):
        try:
            return user.teamspeak3.uid != ''
        except ObjectDoesNotExist:
            return False

    @staticmethod
    @shared_task
    def run_ts3_group_update():
        logger.debug("TS3 installed. Syncing local group objects.")
        with Teamspeak3Manager() as ts3man:
            ts3man._sync_ts_group_db()

    @staticmethod
    def disable():
        logger.info("Deleting all Teamspeak3Users")
        Teamspeak3User.objects.all().delete()
        logger.info("Deleting all UserTSgroup models")
        UserTSgroup.objects.all().delete()
        logger.info("Deleting all AuthTS models")
        AuthTS.objects.all().delete()
        logger.info("Deleting all TSgroup models")
        TSgroup.objects.all().delete()
        logger.info("Teamspeak3 disabled")

    @staticmethod
    @shared_task(bind=True, name="teamspeak3.update_groups", base=QueueOnce)
    def update_groups(self, pk):
        user = User.objects.get(pk=pk)
        logger.debug("Updating user %s teamspeak3 groups" % user)
        if Teamspeak3Tasks.has_account(user):
            usergroups = user.groups.all()
            groups = {}
            for usergroup in usergroups:
                filtered_groups = AuthTS.objects.filter(auth_group=usergroup)
                if filtered_groups:
                    for filtered_group in filtered_groups:
                        for ts_group in filtered_group.ts_group.all():
                            groups[ts_group.ts_group_name] = ts_group.ts_group_id
            for stategroup in user.profile.state.stategroup_set.all():
                groups[stategroup.ts_group.ts_group_name] = stategroup.ts_group.ts_group_id
            logger.debug("Updating user %s teamspeak3 groups to %s" % (user, groups))
            try:
                with Teamspeak3Manager() as ts3man:
                    ts3man.update_groups(user.teamspeak3.uid, groups)
                logger.debug("Updated user %s teamspeak3 groups." % user)
            except TeamspeakError as e:
                logger.error("Error occured while syncing TS groups for %s: %s" % (user, str(e)))
                raise self.retry(countdown=60*10)
        else:
            logger.debug("User does not have a teamspeak3 account")

    @staticmethod
    @shared_task(name="teamspeak3.update_all_groups")
    def update_all_groups():
        logger.debug("Updating ALL teamspeak3 groups")
        for user in Teamspeak3User.objects.exclude(uid__exact=''):
            Teamspeak3Tasks.update_groups.delay(user.user_id)

    @staticmethod
    def get_username(user):
        from .auth_hooks import Teamspeak3Service
        return NameFormatter(Teamspeak3Service(), user).format_name()



from django.test import TestCase

from ..models import EveCharacter, EveCorporationInfo, EveAllianceInfo


class EveCharacterTestCase(TestCase):
    def test_corporation_prop(self):
        """
        Test that the correct corporation is returned by the corporation property
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id='character.alliance.id',
            alliance_name='character.alliance.name',
        )

        expected = EveCorporationInfo.objects.create(
            corporation_id='2345',
            corporation_name='corp.name',
            corporation_ticker='corp.ticker',
            member_count=10,
            alliance=None,
        )

        incorrect = EveCorporationInfo.objects.create(
            corporation_id='9999',
            corporation_name='corp.name1',
            corporation_ticker='corp.ticker1',
            member_count=10,
            alliance=None,
        )

        self.assertEqual(character.corporation, expected)
        self.assertNotEqual(character.corporation, incorrect)

    def test_corporation_prop_exception(self):
        """
        Check that an exception is raised when the expected
        object is not in the database
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id='character.alliance.id',
            alliance_name='character.alliance.name',
        )

        with self.assertRaises(EveCorporationInfo.DoesNotExist):
            result = character.corporation

    def test_alliance_prop(self):
        """
        Test that the correct alliance is returned by the alliance property
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id='3456',
            alliance_name='character.alliance.name',
        )

        expected = EveAllianceInfo.objects.create(
            alliance_id='3456',
            alliance_name='alliance.name',
            alliance_ticker='alliance.ticker',
            executor_corp_id='alliance.executor_corp_id',
        )

        incorrect = EveAllianceInfo.objects.create(
            alliance_id='9001',
            alliance_name='alliance.name1',
            alliance_ticker='alliance.ticker1',
            executor_corp_id='alliance.executor_corp_id1',
        )

        self.assertEqual(character.alliance, expected)
        self.assertNotEqual(character.alliance, incorrect)

    def test_alliance_prop_exception(self):
        """
        Check that an exception is raised when the expected
        object is not in the database
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id='3456',
            alliance_name='character.alliance.name',
        )

        with self.assertRaises(EveAllianceInfo.DoesNotExist):
            result = character.alliance

    def test_alliance_prop_none(self):
        """
        Check that None is returned when the character has no alliance
        """
        character = EveCharacter.objects.create(
            character_id='1234',
            character_name='character.name',
            corporation_id='2345',
            corporation_name='character.corp.name',
            corporation_ticker='character.corp.ticker',
            alliance_id=None,
            alliance_name=None,
        )

        self.assertIsNone(character.alliance)



import logging

from celery import shared_task
from .models import EveAllianceInfo
from .models import EveCharacter
from .models import EveCorporationInfo

logger = logging.getLogger(__name__)


@shared_task
def update_corp(corp_id):
    EveCorporationInfo.objects.update_corporation(corp_id)


@shared_task
def update_alliance(alliance_id):
    EveAllianceInfo.objects.update_alliance(alliance_id).populate_alliance()


@shared_task
def update_character(character_id):
    EveCharacter.objects.update_character(character_id)


@shared_task
def run_model_update():
    # update existing corp models
    for corp in EveCorporationInfo.objects.all().values('corporation_id'):
        update_corp.delay(corp['corporation_id'])

    # update existing alliance models
    for alliance in EveAllianceInfo.objects.all().values('alliance_id'):
        update_alliance.delay(alliance['alliance_id'])

    for character in EveCharacter.objects.all().values('character_id'):
        update_character.delay(character['character_id'])



from unittest import mock

from django.test import TestCase

from allianceauth.tests.auth_utils import AuthUtils

from allianceauth.services.tasks import validate_services


class ServicesTasksTestCase(TestCase):
    def setUp(self):
        self.member = AuthUtils.create_user('auth_member')

    @mock.patch('allianceauth.services.tasks.ServicesHook')
    def test_validate_services(self, services_hook):
        svc = mock.Mock()
        svc.validate_user.return_value = None

        services_hook.get_services.return_value = [svc]

        validate_services.delay(self.member.pk)

        self.assertTrue(services_hook.get_services.called)
        self.assertTrue(svc.validate_user.called)
        args, kwargs = svc.validate_user.call_args
        self.assertEqual(self.member, args[0])  # Assert correct user is passed to service hook function



import logging

logger = logging.getLogger(__name__)


class NotificationHandler(logging.Handler):
    def emit(self, record):
        from django.contrib.auth.models import User, Permission
        from django.db.models import Q
        from . import notify
        from .models import Notification

        try:
            perm = Permission.objects.get(codename="logging_notifications")

            message = record.getMessage()
            if record.exc_text:
                message += "\n\n"
                message = message + record.exc_text

            users = User.objects.filter(
                Q(groups__permissions=perm) | Q(user_permissions=perm) | Q(is_superuser=True)).distinct()

            for user in users:
                notify(
                    user,
                    "%s [%s:%s]" % (record.levelname, record.funcName, record.lineno),
                    level=str([item[0] for item in Notification.LEVEL_CHOICES if item[1] == record.levelname][0]),
                    message=message
                )
        except Permission.DoesNotExist:
            pass



from allianceauth.services.hooks import MenuItemHook, UrlHook

from allianceauth import hooks
from . import urls


class SrpMenu(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(self, 'Ship Replacement',
                              'fa fa-money fa-fw',
                              'srp:management',
                              navactive=['srp:'])

    def render(self, request):
        if request.user.has_perm('srp.access_srp'):
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return SrpMenu()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'srp', r'^srp/')



import requests
import math
from django.conf import settings
from requests_oauthlib import OAuth2Session
from functools import wraps
import logging
import datetime
import time
from django.core.cache import cache
from hashlib import md5

logger = logging.getLogger(__name__)

DISCORD_URL = "https://discordapp.com/api"
EVE_IMAGE_SERVER = "https://image.eveonline.com"

AUTH_URL = "https://discordapp.com/api/oauth2/authorize"
TOKEN_URL = "https://discordapp.com/api/oauth2/token"

"""
Previously all we asked for was permission to kick members, manage roles, and manage nicknames.
Users have reported weird unauthorized errors we don't understand. So now we ask for full server admin.
It's almost fixed the problem.
"""
# kick members, manage roles, manage nicknames, create instant invite
# BOT_PERMISSIONS = 0x00000002 + 0x10000000 + 0x08000000 + 0x00000001
BOT_PERMISSIONS = 0x00000008

# get user ID, accept invite
SCOPES = [
    'identify',
    'guilds.join',
]

GROUP_CACHE_MAX_AGE = getattr(settings, 'DISCORD_GROUP_CACHE_MAX_AGE', 2 * 60 * 60)  # 2 hours default


class DiscordApiException(Exception):
    def __init__(self):
        super(Exception, self).__init__()


class DiscordApiTooBusy(DiscordApiException):
    def __init__(self):
        super(DiscordApiException, self).__init__()
        self.message = "The Discord API is too busy to process this request now, please try again later."


class DiscordApiBackoff(DiscordApiException):
    def __init__(self, retry_after, global_ratelimit):
        """
        :param retry_after: int time to retry after in milliseconds
        :param global_ratelimit: bool Is the API under a global backoff
        """
        super(DiscordApiException, self).__init__()
        self.retry_after = retry_after
        self.global_ratelimit = global_ratelimit

    @property
    def retry_after_seconds(self):
        return math.ceil(self.retry_after / 1000)


cache_time_format = '%Y-%m-%d %H:%M:%S.%f'


def api_backoff(func):
    """
    Decorator, Handles HTTP 429 "Too Many Requests" messages from the Discord API
    If blocking=True is specified, this function will block and retry
    the function up to max_retries=n times, or 3 if retries is not specified.
    If the API call still recieves a backoff timer this function will raise
    a <DiscordApiTooBusy> exception.
    If the caller chooses blocking=False, the decorator will raise a DiscordApiBackoff
    exception and the caller can choose to retry after the given timespan available in
    the retry_after property in seconds.
    """

    class PerformBackoff(Exception):
        def __init__(self, retry_after, retry_datetime, global_ratelimit):
            super(Exception, self).__init__()
            self.retry_after = int(retry_after)
            self.retry_datetime = retry_datetime
            self.global_ratelimit = global_ratelimit

    @wraps(func)
    def decorated(*args, **kwargs):
        blocking = kwargs.get('blocking', False)
        retries = kwargs.get('max_retries', 3)

        # Strip our parameters
        if 'max_retries' in kwargs:
            del kwargs['max_retries']
        if 'blocking' in kwargs:
            del kwargs['blocking']

        cache_key = 'DISCORD_BACKOFF_' + func.__name__
        cache_global_key = 'DISCORD_BACKOFF_GLOBAL'

        while retries > 0:
            try:
                try:
                    # Check global backoff first, then route backoff
                    existing_global_backoff = cache.get(cache_global_key)
                    existing_backoff = existing_global_backoff or cache.get(cache_key)
                    if existing_backoff:
                        backoff_timer = datetime.datetime.strptime(existing_backoff, cache_time_format)
                        if backoff_timer > datetime.datetime.utcnow():
                            backoff_seconds = (backoff_timer - datetime.datetime.utcnow()).total_seconds()
                            logger.debug("Still under backoff for %s seconds, backing off" % backoff_seconds)
                            # Still under backoff
                            raise PerformBackoff(
                                retry_after=backoff_seconds,
                                retry_datetime=backoff_timer,
                                global_ratelimit=bool(existing_global_backoff)
                            )
                    logger.debug("Calling API calling function")
                    return func(*args, **kwargs)
                except requests.HTTPError as e:
                    if e.response.status_code == 429:
                        try:
                            retry_after = int(e.response.headers['Retry-After'])
                        except (TypeError, KeyError):
                            # Pick some random time
                            retry_after = 5000

                        logger.info("Received backoff from API of %s seconds, handling" % retry_after)
                        # Store value in redis
                        backoff_until = (datetime.datetime.utcnow() +
                                         datetime.timedelta(milliseconds=retry_after))
                        global_backoff = bool(e.response.headers.get('X-RateLimit-Global', False))
                        if global_backoff:
                            logger.info("Global backoff!!")
                            cache.set(cache_global_key, backoff_until.strftime(cache_time_format), retry_after)
                        else:
                            cache.set(cache_key, backoff_until.strftime(cache_time_format), retry_after)
                        raise PerformBackoff(retry_after=retry_after, retry_datetime=backoff_until,
                                             global_ratelimit=global_backoff)
                    else:
                        # Not 429, re-raise
                        raise e
            except PerformBackoff as bo:
                # Sleep if we're blocking
                if blocking:
                    logger.info("Blocking Back off from API calls for %s seconds" % bo.retry_after)
                    time.sleep((10 if bo.retry_after > 10 else bo.retry_after) / 1000)
                else:
                    # Otherwise raise exception and let caller handle the backoff
                    raise DiscordApiBackoff(retry_after=bo.retry_after, global_ratelimit=bo.global_ratelimit)
            finally:
                retries -= 1
        if retries == 0:
            raise DiscordApiTooBusy()
    return decorated


class DiscordOAuthManager:
    def __init__(self):
        pass

    @staticmethod
    def _sanitize_name(name):
        return name[:32]

    @staticmethod
    def _sanitize_group_name(name):
        return name[:100]

    @staticmethod
    def generate_bot_add_url():
        return AUTH_URL + '?client_id=' + settings.DISCORD_APP_ID + '&scope=bot&permissions=' + str(BOT_PERMISSIONS)

    @staticmethod
    def generate_oauth_redirect_url():
        oauth = OAuth2Session(settings.DISCORD_APP_ID, redirect_uri=settings.DISCORD_CALLBACK_URL, scope=SCOPES)
        url, state = oauth.authorization_url(AUTH_URL)
        return url

    @staticmethod
    def _process_callback_code(code):
        oauth = OAuth2Session(settings.DISCORD_APP_ID, redirect_uri=settings.DISCORD_CALLBACK_URL)
        token = oauth.fetch_token(TOKEN_URL, client_secret=settings.DISCORD_APP_SECRET, code=code)
        return token

    @staticmethod
    def add_user(code, groups, nickname=None):
        try:
            token = DiscordOAuthManager._process_callback_code(code)['access_token']
            logger.debug("Received token from OAuth")

            custom_headers = {'accept': 'application/json', 'authorization': 'Bearer ' + token}
            path = DISCORD_URL + "/users/@me"
            r = requests.get(path, headers=custom_headers)
            logger.debug("Got status code %s after retrieving Discord profile" % r.status_code)
            r.raise_for_status()

            user_id = r.json()['id']

            path = DISCORD_URL + "/guilds/" + str(settings.DISCORD_GUILD_ID) + "/members/" + str(user_id)
            group_ids = [DiscordOAuthManager._group_name_to_id(DiscordOAuthManager._sanitize_group_name(g)) for g in
                         groups]
            data = {
                'roles': group_ids,
                'access_token': token,
            }
            if nickname:
                data['nick'] = DiscordOAuthManager._sanitize_name(nickname)
            custom_headers['authorization'] = 'Bot ' + settings.DISCORD_BOT_TOKEN
            r = requests.put(path, headers=custom_headers, json=data)
            logger.debug("Got status code %s after joining Discord server" % r.status_code)
            r.raise_for_status()

            logger.info("Added Discord user ID %s to server." % user_id)
            return user_id
        except:
            logger.exception("Failed to add Discord user")
            return None

    @staticmethod
    @api_backoff
    def update_nickname(user_id, nickname):
        nickname = DiscordOAuthManager._sanitize_name(nickname)
        custom_headers = {'content-type': 'application/json', 'authorization': 'Bot ' + settings.DISCORD_BOT_TOKEN}
        data = {'nick': nickname}
        path = DISCORD_URL + "/guilds/" + str(settings.DISCORD_GUILD_ID) + "/members/" + str(user_id)
        r = requests.patch(path, headers=custom_headers, json=data)
        logger.debug("Got status code %s after setting nickname for Discord user ID %s (%s)" % (
            r.status_code, user_id, nickname))
        if r.status_code == 404:
            logger.warn("Discord user ID %s could not be found in server." % user_id)
            return True
        r.raise_for_status()
        return True

    @staticmethod
    def delete_user(user_id):
        try:
            custom_headers = {'accept': 'application/json', 'authorization': 'Bot ' + settings.DISCORD_BOT_TOKEN}
            path = DISCORD_URL + "/guilds/" + str(settings.DISCORD_GUILD_ID) + "/members/" + str(user_id)
            r = requests.delete(path, headers=custom_headers)
            logger.debug("Got status code %s after removing Discord user ID %s" % (r.status_code, user_id))
            if r.status_code == 404:
                logger.warn("Discord user ID %s already left the server." % user_id)
                return True
            r.raise_for_status()
            return True
        except:
            logger.exception("Failed to remove Discord user ID %s" % user_id)
            return False

    @staticmethod
    def _get_groups():
        custom_headers = {'accept': 'application/json', 'authorization': 'Bot ' + settings.DISCORD_BOT_TOKEN}
        path = DISCORD_URL + "/guilds/" + str(settings.DISCORD_GUILD_ID) + "/roles"
        r = requests.get(path, headers=custom_headers)
        logger.debug("Got status code %s after retrieving Discord roles" % r.status_code)
        r.raise_for_status()
        return r.json()

    @staticmethod
    def _generate_cache_role_key(name):
        return 'DISCORD_ROLE_NAME__%s' % md5(str(name).encode('utf-8')).hexdigest()

    @staticmethod
    def _group_name_to_id(name):
        name = DiscordOAuthManager._sanitize_group_name(name)

        def get_or_make_role():
            groups = DiscordOAuthManager._get_groups()
            for g in groups:
                if g['name'] == name:
                    return g['id']
            return DiscordOAuthManager._create_group(name)['id']
        return cache.get_or_set(DiscordOAuthManager._generate_cache_role_key(name), get_or_make_role, GROUP_CACHE_MAX_AGE)

    @staticmethod
    def __generate_role(name, **kwargs):
        custom_headers = {'accept': 'application/json', 'authorization': 'Bot ' + settings.DISCORD_BOT_TOKEN}
        path = DISCORD_URL + "/guilds/" + str(settings.DISCORD_GUILD_ID) + "/roles"
        data = {'name': name}
        data.update(kwargs)
        r = requests.post(path, headers=custom_headers, json=data)
        logger.debug("Received status code %s after generating new role." % r.status_code)
        r.raise_for_status()
        return r.json()

    @staticmethod
    def __edit_role(role_id, **kwargs):
        custom_headers = {'content-type': 'application/json', 'authorization': 'Bot ' + settings.DISCORD_BOT_TOKEN}
        path = DISCORD_URL + "/guilds/" + str(settings.DISCORD_GUILD_ID) + "/roles/" + str(role_id)
        r = requests.patch(path, headers=custom_headers, json=kwargs)
        logger.debug("Received status code %s after editing role id %s" % (r.status_code, role_id))
        r.raise_for_status()
        return r.json()

    @staticmethod
    def _create_group(name):
        return DiscordOAuthManager.__generate_role(name)

    @staticmethod
    def _get_user(user_id):
        custom_headers = {'content-type': 'application/json', 'authorization': 'Bot ' + settings.DISCORD_BOT_TOKEN}
        path = DISCORD_URL + "/guilds/" + str(settings.DISCORD_GUILD_ID) + "/members/" + str(user_id)
        r = requests.get(path, headers=custom_headers)
        r.raise_for_status()
        return r.json()

    @staticmethod
    def _get_user_roles(user_id):
        user = DiscordOAuthManager._get_user(user_id)
        return user['roles']

    @staticmethod
    def _modify_user_role(user_id, role_id, method):
        custom_headers = {'content-type': 'application/json', 'authorization': 'Bot ' + settings.DISCORD_BOT_TOKEN}
        path = DISCORD_URL + "/guilds/" + str(settings.DISCORD_GUILD_ID) + "/members/" + str(user_id) + "/roles/" + str(
            role_id)
        r = getattr(requests, method)(path, headers=custom_headers)
        r.raise_for_status()
        logger.debug("%s role %s for user %s" % (method, role_id, user_id))

    @staticmethod
    @api_backoff
    def update_groups(user_id, groups):
        group_ids = [DiscordOAuthManager._group_name_to_id(DiscordOAuthManager._sanitize_group_name(g)) for g in groups]
        user_group_ids = DiscordOAuthManager._get_user_roles(user_id)
        for g in group_ids:
            if g not in user_group_ids:
                DiscordOAuthManager._modify_user_role(user_id, g, 'put')
                time.sleep(1)  # we're gonna be hammering the API here
        for g in user_group_ids:
            if g not in group_ids:
                DiscordOAuthManager._modify_user_role(user_id, g, 'delete')
                time.sleep(1)



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import permission_required
from django.contrib.auth.decorators import user_passes_test
from django.shortcuts import redirect

from allianceauth.services.views import superuser_test
from .manager import DiscordOAuthManager
from .tasks import DiscordTasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'discord.access_discord'


@login_required
@permission_required(ACCESS_PERM)
def deactivate_discord(request):
    logger.debug("deactivate_discord called by user %s" % request.user)
    if DiscordTasks.delete_user(request.user):
        logger.info("Successfully deactivated discord for user %s" % request.user)
        messages.success(request, 'Deactivated Discord account.')
    else:
        logger.error("Unsuccessful attempt to deactivate discord for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your Discord account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_discord(request):
    logger.debug("reset_discord called by user %s" % request.user)
    if DiscordTasks.delete_user(request.user):
        logger.info("Successfully deleted discord user for user %s - forwarding to discord activation." % request.user)
        return redirect("discord:activate")
    logger.error("Unsuccessful attempt to reset discord for user %s" % request.user)
    messages.error(request, 'An error occurred while processing your Discord account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def activate_discord(request):
    logger.debug("activate_discord called by user %s" % request.user)
    return redirect(DiscordOAuthManager.generate_oauth_redirect_url())


@login_required
@permission_required(ACCESS_PERM)
def discord_callback(request):
    logger.debug("Received Discord callback for activation of user %s" % request.user)
    code = request.GET.get('code', None)
    if not code:
        logger.warn("Did not receive OAuth code from callback of user %s" % request.user)
        return redirect("services:services")
    if DiscordTasks.add_user(request.user, code):
        logger.info("Successfully activated Discord for user %s" % request.user)
        messages.success(request, 'Activated Discord account.')
    else:
        logger.error("Failed to activate Discord for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your Discord account.')
    return redirect("services:services")


@login_required
@user_passes_test(superuser_test)
def discord_add_bot(request):
    return redirect(DiscordOAuthManager.generate_bot_add_url())



from allianceauth.services.hooks import MenuItemHook, UrlHook

from allianceauth import hooks
from . import urls


class OpTimerboardMenu(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(self, 'Fleet Operations',
                              'fa fa-exclamation  fa-fw',
                              'optimer:view',
                              navactive=['optimer:'])

    def render(self, request):
        if request.user.has_perm('auth.optimer_view'):
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return OpTimerboardMenu()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'optimer', r'^optimer/')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-06 23:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='groupdescription',
            name='group',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='auth.Group'),
        ),
        migrations.AlterField(
            model_name='hiddengroup',
            name='group',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='auth.Group'),
        ),
    ]



import logging

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from allianceauth.notifications import notify
from celery import shared_task
from allianceauth.services.tasks import QueueOnce
from allianceauth.services.modules.openfire.manager import OpenfireManager
from allianceauth.services.hooks import NameFormatter
from .models import OpenfireUser

logger = logging.getLogger(__name__)


class OpenfireTasks:
    def __init__(self):
        pass

    @classmethod
    def delete_user(cls, user, notify_user=False):
        if cls.has_account(user):
            logger.debug("User %s has jabber account %s. Deleting." % (user, user.openfire.username))
            OpenfireManager.delete_user(user.openfire.username)
            user.openfire.delete()
            if notify_user:
                notify(user, 'Jabber Account Disabled', level='danger')
            return True
        return False

    @staticmethod
    def has_account(user):
        try:
            return user.openfire.username != ''
        except ObjectDoesNotExist:
            return False

    @staticmethod
    def disable_jabber():
        logging.debug("Deleting all Openfire users")
        OpenfireUser.objects.all().delete()

    @staticmethod
    @shared_task(bind=True, name="openfire.update_groups", base=QueueOnce)
    def update_groups(self, pk):
        user = User.objects.get(pk=pk)
        logger.debug("Updating jabber groups for user %s" % user)
        if OpenfireTasks.has_account(user):
            groups = [user.profile.state.name]
            for group in user.groups.all():
                groups.append(str(group.name))
            logger.debug("Updating user %s jabber groups to %s" % (user, groups))
            try:
                OpenfireManager.update_user_groups(user.openfire.username, groups)
            except:
                logger.exception("Jabber group sync failed for %s, retrying in 10 mins" % user)
                raise self.retry(countdown=60 * 10)
            logger.debug("Updated user %s jabber groups." % user)
        else:
            logger.debug("User does not have an openfire account")

    @staticmethod
    @shared_task(name="openfire.update_all_groups")
    def update_all_groups():
        logger.debug("Updating ALL jabber groups")
        for openfire_user in OpenfireUser.objects.exclude(username__exact=''):
            OpenfireTasks.update_groups.delay(openfire_user.user.pk)

    @staticmethod
    def get_username(user):
        from .auth_hooks import OpenfireService
        return NameFormatter(OpenfireService(), user).format_name()



from django import forms
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from .providers import ObjectNotFound

from .models import EveAllianceInfo
from .models import EveCharacter
from .models import EveCorporationInfo


class EveEntityExistsError(forms.ValidationError):
    def __init__(self, entity_type_name, id):
        super(EveEntityExistsError, self).__init__(
            message='{} with ID {} already exists.'.format(entity_type_name, id))


class EveEntityNotFoundError(forms.ValidationError):
    def __init__(self, entity_type_name, id):
        super(EveEntityNotFoundError, self).__init__(
            message='{} with ID {} not found.'.format(entity_type_name, id))


class EveEntityForm(forms.ModelForm):
    id = forms.IntegerField(min_value=1)
    entity_model_class = None

    def clean_id(self):
        raise NotImplementedError()

    def save(self, commit=True):
        raise NotImplementedError()

    def save_m2m(self):
        pass


class EveCharacterForm(EveEntityForm):
    entity_type_name = 'character'

    def clean_id(self):
        try:
            assert self.Meta.model.provider.get_character(self.cleaned_data['id'])
        except (AssertionError, ObjectNotFound):
            raise EveEntityNotFoundError(self.entity_type_name, self.cleaned_data['id'])
        if self.Meta.model.objects.filter(character_id=self.cleaned_data['id']).exists():
            raise EveEntityExistsError(self.entity_type_name, self.cleaned_data['id'])
        return self.cleaned_data['id']

    def save(self, commit=True):
        return self.Meta.model.objects.create_character(self.cleaned_data['id'])

    class Meta:
        fields = ['id']
        model = EveCharacter


class EveCorporationForm(EveEntityForm):
    entity_type_name = 'corporation'

    def clean_id(self):
        try:
            assert self.Meta.model.provider.get_corporation(self.cleaned_data['id'])
        except (AssertionError, ObjectNotFound):
            raise EveEntityNotFoundError(self.entity_type_name, self.cleaned_data['id'])
        if self.Meta.model.objects.filter(corporation_id=self.cleaned_data['id']).exists():
            raise EveEntityExistsError(self.entity_type_name, self.cleaned_data['id'])
        return self.cleaned_data['id']

    def save(self, commit=True):
        return self.Meta.model.objects.create_corporation(self.cleaned_data['id'])

    class Meta:
        fields = ['id']
        model = EveCorporationInfo


class EveAllianceForm(EveEntityForm):
    entity_type_name = 'alliance'

    def clean_id(self):
        try:
            assert self.Meta.model.provider.get_alliance(self.cleaned_data['id'])
        except (AssertionError, ObjectNotFound):
            raise EveEntityNotFoundError(self.entity_type_name, self.cleaned_data['id'])
        if self.Meta.model.objects.filter(alliance_id=self.cleaned_data['id']).exists():
            raise EveEntityExistsError(self.entity_type_name, self.cleaned_data['id'])
        return self.cleaned_data['id']

    def save(self, commit=True):
        return self.Meta.model.objects.create_alliance(self.cleaned_data['id'])

    class Meta:
        fields = ['id']
        model = EveAllianceInfo


@admin.register(EveCorporationInfo)
class EveCorporationInfoAdmin(admin.ModelAdmin):
    def get_form(self, request, obj=None, **kwargs):
        if not obj or not obj.pk:
            return EveCorporationForm
        return super(EveCorporationInfoAdmin, self).get_form(request, obj=obj, **kwargs)


@admin.register(EveAllianceInfo)
class EveAllianceInfoAdmin(admin.ModelAdmin):
    def get_form(self, request, obj=None, **kwargs):
        if not obj or not obj.pk:
            return EveAllianceForm
        return super(EveAllianceInfoAdmin, self).get_form(request, obj=obj, **kwargs)


@admin.register(EveCharacter)
class EveCharacterAdmin(admin.ModelAdmin):
    search_fields = ['character_name', 'corporation_name', 'alliance_name', 'character_ownership__user__username']
    list_display = ('character_name', 'corporation_name', 'alliance_name', 'user', 'main_character')

    @staticmethod
    def user(obj):
        try:
            return obj.character_ownership.user
        except (AttributeError, ObjectDoesNotExist):
            return None

    @staticmethod
    def main_character(obj):
        try:
            return obj.character_ownership.user.profile.main_character
        except (AttributeError, ObjectDoesNotExist):
            return None

    def get_form(self, request, obj=None, **kwargs):
        if not obj or not obj.pk:
            return EveCharacterForm
        return super(EveCharacterAdmin, self).get_form(request, obj=obj, **kwargs)



from django import forms
from django.utils.translation import ugettext_lazy as _


class OpForm(forms.Form):
    doctrine = forms.CharField(max_length=254, required=True, label=_('Doctrine'))
    system = forms.CharField(max_length=254, required=True, label=_("System"))
    start = forms.DateTimeField(required=True, label=_("Start Time"))
    duration = forms.CharField(max_length=254, required=True, label=_("Duration"))
    operation_name = forms.CharField(max_length=254, required=True, label=_("Operation Name"))
    fc = forms.CharField(max_length=254, required=True, label=_("Fleet Commander"))



from django.apps import AppConfig


class Teamspeak3ServiceConfig(AppConfig):
    name = 'allianceauth.services.modules.teamspeak3'
    label = 'teamspeak3'

    def ready(self):
        from . import signals



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-05 21:40
from __future__ import unicode_literals

import datetime
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eveonline', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='optimer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('doctrine', models.CharField(default=b'', max_length=254)),
                ('system', models.CharField(default=b'', max_length=254)),
                ('location', models.CharField(default=b'', max_length=254)),
                ('start', models.DateTimeField(default=datetime.datetime.now)),
                ('duration', models.CharField(default=b'', max_length=25)),
                ('operation_name', models.CharField(default=b'', max_length=254)),
                ('fc', models.CharField(default=b'', max_length=254)),
                ('details', models.CharField(default=b'', max_length=254)),
                ('post_time', models.DateTimeField(default=django.utils.timezone.now)),
                ('eve_character', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveonline.EveCharacter')),
            ],
            options={
                'ordering': ['start'],
            },
        ),
    ]



from . import views

from django.conf.urls import include, url
app_name = 'groupmanagement'

urlpatterns = [
    url(r'^groups/', views.groups_view, name='groups'),
    url(r'^group/', include([
        url(r'^management/', views.group_management,
            name='management'),
        url(r'^membership/$', views.group_membership,
            name='membership'),
        url(r'^membership/(\w+)/$', views.group_membership_list,
            name='membership_list'),
        url(r'^membership/(\w+)/remove/(\w+)/$', views.group_membership_remove,
            name='membership_remove'),
        url(r'^request_add/(\w+)', views.group_request_add,
            name='request_add'),
        url(r'^request/accept/(\w+)', views.group_accept_request,
            name='accept_request'),
        url(r'^request/reject/(\w+)', views.group_reject_request,
            name='reject_request'),

        url(r'^request_leave/(\w+)', views.group_request_leave,
            name='request_leave'),
        url(r'leave_request/accept/(\w+)', views.group_leave_accept_request,
            name='leave_accept_request'),
        url(r'^leave_request/reject/(\w+)', views.group_leave_reject_request,
            name='leave_reject_request'),
    ])),
]



from django.contrib import admin

from allianceauth.srp.models import SrpFleetMain
from allianceauth.srp.models import SrpUserRequest

admin.site.register(SrpFleetMain)
admin.site.register(SrpUserRequest)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-09 23:22
from __future__ import unicode_literals

from django.db import migrations
from django.conf import settings

class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0002_auto_20160906_2354'),
    ]

    operations = []



#!/usr/bin/env python
import os
import shutil
from optparse import OptionParser
from django.core.management import call_command
from django.core.management.commands.startproject import Command as BaseStartProject


class StartProject(BaseStartProject):
    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument('--python', help='The path to the python executable.')
        parser.add_argument('--celery', help='The path to the celery executable.')
        parser.add_argument('--gunicorn', help='The path to the gunicorn executable.')


def create_project(parser, options, args):
    # Validate args
    if len(args) < 2:
        parser.error("Please specify a name for your Alliance Auth installation.")
    elif len(args) > 3:
        parser.error("Too many arguments.")

    # First find the path to Alliance Auth
    import allianceauth
    allianceauth_path = os.path.dirname(allianceauth.__file__)
    template_path = os.path.join(allianceauth_path, 'project_template')

    # Determine locations of commands to render supervisor cond
    command_options = {
        'template': template_path,
        'python': shutil.which('python'),
        'gunicorn': shutil.which('gunicorn'),
        'celery': shutil.which('celery'),
        'extensions': ['py', 'conf', 'json'],
    }

    # Strip 'start' out of the arguments, leaving project name (and optionally destination dir)
    args = args[1:]

    # Call the command with extra context
    call_command(StartProject(), *args, **command_options)

    print("Success! %(project_name)s has been created." % {'project_name': args[0]})  # noqa


def update_settings(parser, options, args):
    if len(args) < 2:
        parser.error("Please specify the path to your Alliance Auth installation.")
    elif len(args) > 2:
        parser.error("Too many arguments.")

    project_path = args[1]
    project_name = os.path.split(project_path)[-1]

    # find the target settings/base.py file, handing both the project and app as valid paths
    # first check if given path is to the app
    settings_path = os.path.join(project_path, 'settings/base.py')
    if not os.path.exists(settings_path):
        # next check if given path is to the project, so the app is within it
        settings_path = os.path.join(project_path, project_name, 'settings/base.py')
        if not os.path.exists(settings_path):
            parser.error("Unable to locate the Alliance Auth project at %s" % project_path)

    # first find the path to the Alliance Auth template settings
    import allianceauth
    allianceauth_path = os.path.dirname(allianceauth.__file__)
    template_path = os.path.join(allianceauth_path, 'project_template')
    template_settings_path = os.path.join(template_path, 'project_name/settings/base.py')

    # overwrite the local project's base settings
    with open(template_settings_path, 'r') as template, open(settings_path, 'w') as target:
        target.write(template.read())

    print("Successfully updated %(project_name)s settings." % {'project_name': project_name})


COMMANDS = {
    'start': create_project,
    'update': update_settings,
}


def main():
    # Parse options
    parser = OptionParser(usage="Usage: %prog [start|update] project_name [directory]")
    (options, args) = parser.parse_args()

    # Find command
    try:
        command = args[0]
    except IndexError:
        parser.print_help()
        return

    if command in COMMANDS:
        COMMANDS[command](parser, options, args)
    else:
        parser.error("Unrecognised command: " + command)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-02 05:59
from __future__ import unicode_literals

from django.db import migrations
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.management import create_permissions

import logging

logger = logging.getLogger(__name__)


def migrate_service_enabled(apps, schema_editor):
    for app_config in apps.get_app_configs():
        app_config.models_module = True
        create_permissions(app_config, apps=apps, verbosity=0)
        app_config.models_module = None

    Group = apps.get_model("auth", "Group")
    Permission = apps.get_model("auth", "Permission")
    MumbleUser = apps.get_model("mumble", "MumbleUser")

    perm = Permission.objects.get(codename='access_mumble')

    member_group_name = getattr(settings, str('DEFAULT_AUTH_GROUP'), 'Member')
    blue_group_name = getattr(settings, str('DEFAULT_BLUE_GROUP'), 'Blue')

    # Migrate members
    if MumbleUser.objects.filter(user__groups__name=member_group_name).exists() or \
            getattr(settings, str('ENABLE_AUTH_MUMBLE'), False):
        try:
            group = Group.objects.get(name=member_group_name)
            group.permissions.add(perm)
        except ObjectDoesNotExist:
            logger.warning('Failed to migrate ENABLE_AUTH_MUMBLE setting')

    # Migrate blues
    if MumbleUser.objects.filter(user__groups__name=blue_group_name).exists() or \
            getattr(settings, str('ENABLE_BLUE_MUMBLE'), False):
        try:
            group = Group.objects.get(name=blue_group_name)
            group.permissions.add(perm)
        except ObjectDoesNotExist:
            logger.warning('Failed to migrate ENABLE_BLUE_MUMBLE setting')


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0005_mumbleuser_hashfn'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='mumbleuser',
            options={'permissions': (('access_mumble', 'Can access the Mumble service'),)},
        ),
        migrations.RunPython(migrate_service_enabled),
    ]



from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect

from .manager import DiscourseManager
from .tasks import DiscourseTasks
from .models import DiscourseUser

import base64
import hmac
import hashlib

try:
    from urllib import unquote, urlencode
except ImportError: #py3
    from urllib.parse import unquote, urlencode
try:
    from urlparse import parse_qs
except ImportError: #py3
    from urllib.parse import parse_qs

import logging

logger = logging.getLogger(__name__)


ACCESS_PERM = 'discourse.access_discourse'


@login_required
def discourse_sso(request):

    # Check if user has access
    if not request.user.has_perm(ACCESS_PERM):
        messages.error(request, 'You are not authorized to access Discourse.')
        logger.warning('User %s attempted to access Discourse but does not have permission.' % request.user)
        return redirect('authentication:dashboard')

    if not request.user.profile.main_character:
        messages.error(request, "You must have a main character set to access Discourse.")
        logger.warning('User %s attempted to access Discourse but does not have a main character.' % request.user)
        return redirect('authentication:characters')

    main_char = request.user.profile.main_character

    payload = request.GET.get('sso')
    signature = request.GET.get('sig')

    if None in [payload, signature]:
        messages.error(request, 'No SSO payload or signature. Please contact support if this problem persists.')
        return redirect('authentication:dashboard')

    # Validate the payload
    try:
        payload = unquote(payload).encode('utf-8')
        decoded = base64.decodestring(payload).decode('utf-8')
        assert 'nonce' in decoded
        assert len(payload) > 0
    except AssertionError:
        messages.error(request, 'Invalid payload. Please contact support if this problem persists.')
        return redirect('authentication:dashboard')

    key = str(settings.DISCOURSE_SSO_SECRET).encode('utf-8')
    h = hmac.new(key, payload, digestmod=hashlib.sha256)
    this_signature = h.hexdigest()

    if this_signature != signature:
        messages.error(request, 'Invalid payload. Please contact support if this problem persists.')
        return redirect('authentication:dashboard')

    ## Build the return payload

    username = DiscourseManager._sanitize_username(DiscourseTasks.get_username(request.user))

    qs = parse_qs(decoded)
    params = {
        'nonce': qs['nonce'][0],
        'email': request.user.email,
        'external_id': request.user.pk,
        'username': username,
        'name': username,
    }

    if main_char:
        params['avatar_url'] = 'https://image.eveonline.com/Character/%s_256.jpg' % main_char.character_id

    return_payload = base64.encodestring(urlencode(params).encode('utf-8'))
    h = hmac.new(key, return_payload, digestmod=hashlib.sha256)
    query_string = urlencode({'sso': return_payload, 'sig': h.hexdigest()})

    # Record activation and queue group sync
    if not DiscourseTasks.has_account(request.user):
        discourse_user = DiscourseUser()
        discourse_user.user = request.user
        discourse_user.enabled = True
        discourse_user.save()
        # wait 30s for new user creation on Discourse before triggering group sync
        DiscourseTasks.update_groups.apply_async(args=[request.user.pk], countdown=30)

    # Redirect back to Discourse
    url = '%s/session/sso_login' % settings.DISCOURSE_URL
    return redirect('%s?%s' % (url, query_string))




# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-05 21:40
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DiscordAuthToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.CharField(max_length=254, unique=True)),
                ('token', models.CharField(max_length=254)),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='GroupCache',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('groups', models.TextField(default={})),
                ('service', models.CharField(choices=[(b'discourse', b'discourse'), (b'discord', b'discord')], max_length=254, unique=True)),
            ],
        ),
    ]



from django.db import models
from django.contrib.auth.models import User
import logging

logger = logging.getLogger(__name__)


class Notification(models.Model):
    LEVEL_CHOICES = (
        ('danger', 'CRITICAL'),
        ('danger', 'ERROR'),
        ('warning', 'WARN'),
        ('info', 'INFO'),
        ('success', 'DEBUG'),
    )

    user = models.ForeignKey(User, on_delete=models.CASCADE)
    level = models.CharField(choices=LEVEL_CHOICES, max_length=10)
    title = models.CharField(max_length=254)
    message = models.TextField()
    timestamp = models.DateTimeField(auto_now_add=True)
    viewed = models.BooleanField(default=False)

    def view(self):
        logger.info("Marking notification as viewed: %s" % self)
        self.viewed = True
        self.save()

    def __str__(self):
        return "%s: %s" % (self.user, self.title)

    def set_level(self, level):
        self.level = [item[0] for item in self.LEVEL_CHOICES if item[1] == level][0]

    class Meta:
        ordering = ['-timestamp']
